@@ -44,6 +44,11 @@ class AudioManager : public QObject
     Q_PROPERTY(QString formattedPosition READ formattedPosition NOTIFY positionChanged)
 
 public:
+    const double MAX_RATE = 1.0;
+    const double MIN_RATE = 2.5;
+    const qint64 SKIP_STEP = 10000;
+    const qint64 SKIP_TRACK_END = 15000;
+
     static AudioManager &instance()
     {
         static AudioManager _instance;
