@@ -10,9 +10,16 @@
 
 #include <KLocalizedString>
 #include <QFile>
+#include <QFileInfo>
+#include <QMimeDatabase>
 #include <QNetworkReply>
 #include <QSqlQuery>
 
+#include <attachedpictureframe.h>
+#include <id3v2frame.h>
+#include <id3v2tag.h>
+#include <mpegfile.h>
+
 #include "audiomanager.h"
 #include "database.h"
 #include "datamanager.h"
@@ -32,8 +39,18 @@ Enclosure::Enclosure(Entry *entry)
     : QObject(entry)
     , m_entry(entry)
 {
+    connect(this, &Enclosure::playPositionChanged, this, &Enclosure::leftDurationChanged);
     connect(this, &Enclosure::statusChanged, &DownloadModel::instance(), &DownloadModel::monitorDownloadStatus);
     connect(this, &Enclosure::downloadError, &ErrorLogModel::instance(), &ErrorLogModel::monitorErrorMessages);
+    connect(&Fetcher::instance(), &Fetcher::entryUpdated, this, [this](const QString &url, const QString &id) {
+        if ((m_entry->feed()->url() == url) && (m_entry->id() == id)) {
+            updateFromDb();
+        }
+    });
+
+    // we use the relayed signal from AudioManager::playbackRateChanged by
+    // DataManager; this is required to avoid a dependency loop on startup
+    connect(&DataManager::instance(), &DataManager::playbackRateChanged, this, &Enclosure::leftDurationChanged);
 
     QSqlQuery query;
     query.prepare(QStringLiteral("SELECT * FROM Enclosures WHERE id=:id"));
@@ -56,6 +73,51 @@ Enclosure::Enclosure(Entry *entry)
     checkSizeOnDisk();
 }
 
+void Enclosure::updateFromDb()
+{
+    // This method is used to update the most relevant fields from the RSS feed,
+    // most notably the download URL.  It's deliberatly only updating the
+    // duration and size if the URL has changed, since these values are
+    // notably untrustworthy.  We generally get them from the files themselves
+    // at the time they are downloaded.
+    QSqlQuery query;
+    query.prepare(QStringLiteral("SELECT * FROM Enclosures WHERE id=:id"));
+    query.bindValue(QStringLiteral(":id"), m_entry->id());
+    Database::instance().execute(query);
+
+    if (!query.next()) {
+        return;
+    }
+
+    if (m_url != query.value(QStringLiteral("url")).toString() && m_status != Downloaded) {
+        // this means that the audio file has changed, or at least its location
+        // let's only do something if the file isn't downloaded.
+        // try to delete the file first (it actually shouldn't exist)
+        deleteFile();
+
+        m_url = query.value(QStringLiteral("url")).toString();
+        Q_EMIT urlChanged(m_url);
+        Q_EMIT pathChanged(path());
+
+        if (m_duration != query.value(QStringLiteral("duration")).toInt()) {
+            m_duration = query.value(QStringLiteral("duration")).toInt();
+            Q_EMIT durationChanged();
+        }
+        if (m_size != query.value(QStringLiteral("size")).toInt()) {
+            m_size = query.value(QStringLiteral("size")).toInt();
+            Q_EMIT sizeChanged();
+        }
+        if (m_title != query.value(QStringLiteral("title")).toString()) {
+            m_title = query.value(QStringLiteral("title")).toString();
+            Q_EMIT titleChanged(m_title);
+        }
+        if (m_type != query.value(QStringLiteral("type")).toString()) {
+            m_type = query.value(QStringLiteral("type")).toString();
+            Q_EMIT typeChanged(m_type);
+        }
+    }
+}
+
 int Enclosure::statusToDb(Enclosure::Status status)
 {
     switch (status) {
@@ -237,6 +299,50 @@ Enclosure::Status Enclosure::status() const
     return m_status;
 }
 
+QString Enclosure::cachedEmbeddedImage() const
+{
+    // if image is already cached, then return the path
+    QString cachedpath = StorageManager::instance().imagePath(path());
+    if (QFileInfo::exists(cachedpath)) {
+        if (QFileInfo(cachedpath).size() != 0) {
+            return QUrl::fromLocalFile(cachedpath).toString();
+        }
+    }
+
+    if (m_status != Downloaded || path().isEmpty()) {
+        return QStringLiteral("");
+    }
+
+    const auto mime = QMimeDatabase().mimeTypeForFile(path()).name();
+    if (mime != QStringLiteral("audio/mpeg")) {
+        return QStringLiteral("");
+    }
+
+    TagLib::MPEG::File f(path().toLatin1().data());
+    if (!f.hasID3v2Tag()) {
+        return QStringLiteral("");
+    }
+
+    bool imageFound = false;
+    for (const auto &frame : f.ID3v2Tag()->frameListMap()["APIC"]) {
+        auto pictureFrame = dynamic_cast<TagLib::ID3v2::AttachedPictureFrame *>(frame);
+        QByteArray data(pictureFrame->picture().data(), pictureFrame->picture().size());
+        if (!data.isEmpty()) {
+            QFile file(cachedpath);
+            file.open(QIODevice::WriteOnly);
+            file.write(data);
+            file.close();
+            imageFound = true;
+        }
+    }
+
+    if (imageFound) {
+        return cachedpath;
+    } else {
+        return QUrl::fromLocalFile(cachedpath).toString();
+    }
+}
+
 qint64 Enclosure::playPosition() const
 {
     return m_playposition;
@@ -388,7 +494,13 @@ QString Enclosure::formattedDuration() const
 
 QString Enclosure::formattedLeftDuration() const
 {
-    return m_kformat.formatDuration(duration() * 1000 - playPosition());
+    qreal rate = 1.0;
+    if (SettingsManager::self()->adjustTimeLeft()) {
+        rate = AudioManager::instance().playbackRate();
+        rate = (rate > 0.0) ? rate : 1.0;
+    }
+    qint64 diff = duration() * 1000 - playPosition();
+    return m_kformat.formatDuration(diff / rate);
 }
 
 QString Enclosure::formattedPlayPosition() const
